;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; See http://www.imada.ou.dk/Technical/Manpages/emacs/templates/templates.html
;;; for documentation
;;;
;;; When this library is loaded, Emacs will search for a template
;;; when you create a new file. It will search for a file named
;;; <prefix>.<ext> where prefix is given with the variable
;;; template-prefix below, and ext is the extension of the file
;;; you create.
;;; Emacs will search for this file in the current directory
;;; if it doesn't find it there, it will search for it in the
;;; directory above, and so on.
;;; Jesper K. Pedersen (blackie@imada.ou.dk)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defvar template-prefix ".template")

;;; call ths function LT-insert-template whenever a
;;; new file is opened.
(add-hook 'find-file-hooks 'LT-insert-template)
      

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; This function search for a template file and insert it into
;;; the current buffer, if it is empty.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun LT-insert-template ()
  (interactive)
  (let* ((file-name (buffer-file-name))
         (size (buffer-size)))
    (if (and (not (null file-name)) (= 0 size))
        (let* ((extension (LT-file-name-extension file-name))
               (path (file-name-directory file-name))
               (template-file (LT-find-template-file extension path)))
          (if template-file
              (LT-insert-the-template template-file))))))


;;; This function returns the extension of a file name
(defun LT-file-name-extension (file-name)
  (if file-name
      (let* ( (name (file-name-nondirectory file-name))
              (no-ver (file-name-sans-versions file-name))
              (dummy (string-match ".*\\.\\([^.]*\\)$" no-ver))
              (match (match-beginning 1)))
        (if (null match) ""  (substring no-ver match)))))

;;; This function search for a template file for
;;; 'extension' in the directory paths 'path'
(defun LT-find-template-file (extension path)
  (let ((name (concat path template-prefix "." extension)))
    (if path
        (if (and (file-exists-p name) (file-readable-p name))
            name
          (LT-find-template-file
           extension (file-name-directory  (substring path 0 -1)))) 
    nil)))


;; given a file name, insert the templates located in this file into the
;; current buffer.
(defun LT-insert-the-template (file-name)
  (interactive "f")
  (let ((whole-buffer (get-buffer-create "*LT-whole*"))
        (template-buffer (buffer-name))
        (expansion-buffer (get-buffer-create "*LT-expansions*"))
        ; make variables local
        (beginning-of-buffer nil)
        (end-of-buffer nil)
        (expansion-end nil)
        (expansion-start nil)
        (expansion-list '()))

    ;; insert the file
    (set-buffer whole-buffer)
    (insert-file-contents file-name)
    (goto-char (point-min))

    ;; search for the beginning of the different regions
    (setq beginning-of-buffer (point-min))
    (setq end-of-buffer (point-max))
    (if (or (search-forward-regexp "^< *-* *LT expansion *-* *>$" nil t)
            (search-forward-regexp "< *-* *>" nil t))
        (progn
          (setq expansion-end (match-end 0))
          (setq expansion-start (match-beginning 0)))
      (setq expansion-start nil))


    ; copy template segment to other buffer
    (set-buffer template-buffer)
    (if expansion-start
        (insert-buffer-substring whole-buffer beginning-of-buffer
                                 expansion-start)
      (insert-buffer-substring whole-buffer))

    ; copy expansion segment to other buffer
    (if expansion-start
        (progn
          (set-buffer expansion-buffer)
          (insert "()")
          (forward-char -1)
          (insert-buffer-substring whole-buffer expansion-end end-of-buffer)))

  
    ; read the expansions into a file
    (set-buffer expansion-buffer)
    (if expansion-start
        (progn
          (beginning-of-buffer)
          (setq expansion-list (read expansion-buffer))
          (erase-buffer)))
    
    (if (and (file-exists-p "~/.template-expansions")
             (file-readable-p "~/.template-expansions"))
        (progn
          (insert "()")
          (forward-char -1)
          (insert-file "~/.template-expansions")
          (beginning-of-buffer)
          (setq expansion-list
                (append expansion-list (read expansion-buffer)))))

    ; apply the expansion
    (set-buffer template-buffer)
    (while expansion-list
      (let ((place-holder (car expansion-list))
            (replacement (car (cdr expansion-list))))
        (setq expansion-list (cdr (cdr expansion-list)))
        (beginning-of-buffer)
        (if (search-forward place-holder nil t)
            (progn
              (beginning-of-buffer)
              (setq replacement
                    (if (stringp replacement)
                        (read-string replacement)
                      (eval replacement)))
                    
              (while (search-forward place-holder nil t)
                (replace-match replacement t t))))))
          
    ; place the pointer
    (beginning-of-buffer)
    (if (search-forward "@POINT@" nil t)
        (progn
          (replace-match "" t t)
          (goto-char (match-beginning 0))))
      
    ; delete the temporary buffers
    (kill-buffer whole-buffer)
    (kill-buffer expansion-buffer)
    ))

;;; This function ask the yes/no quesion 'question', and evaluate
;;; 'yes' if yes were ansered to the question otherwise it evaluates 'no'
(defun y-n-question (question yes no)
  (if (y-or-n-p question)
      (if (stringp yes)
          yes
        (eval yes))
    (if (stringp no)
        no
      (eval no))))

(defun question-with-default-value (question default)
  (let ((replacement (read-string question)))
    (if (not (equal replacement ""))
        replacement
      default)))
      
                     
    
;;; Local Variables:
;;; eval: (defun byte-compile-this-file () (write-region (point-min) (point-max) buffer-file-name nil 't) (byte-compile-file buffer-file-name) nil)
;;; write-file-hooks: (byte-compile-this-file)
;;; End:
